use serde::{Deserialize, Serialize};
use sea_orm::entity::prelude::*;


// Model是只读的，只能用来SELECT
#[derive(Debug, Clone, PartialEq, DeriveEntityModel, Serialize, Deserialize)]
#[sea_orm(table_name = "articles")]
pub struct Model {
    #[sea_orm(primary_key)]
    #[serde(skip_deserializing)]
    pub id: u32,
    pub category_id: u32,
    pub title: String,
    pub content: String,
    pub dateline: chrono::DateTime<chrono::Local>,
    pub is_del: bool,
}

#[derive(Debug, Clone, Copy, EnumIter)]
pub enum Relation {
    // 给文章实体定义了包含Category的关系：
    Category,
}

impl RelationTrait for Relation {
    // 定义了 Relation 并没实际意义，毕竟它只是一个枚举值而已。我们需要为这个关系实现业务逻辑：
    fn def(&self) -> RelationDef {
        return match self {
            // 声明当前模型属于category模型(多对一)，从category_id关联对应表的id字段
            Relation::Category => {
                Entity::belongs_to(super::category::Entity)
                    .from(Column::CategoryId)
                    .to(super::category::Column::Id)
                    .into()
            }
        }
    }
}

// 定义好了关系，我们还需要实现 Relate：告诉 SeaORM ，我们定义的 Relation::Category关系需要如何去建立
impl Related<super::category::Entity> for Entity {
    fn to() -> RelationDef {
        Relation::Category.def()
    }
}

// INSERT/UPDATE/DELETE 等属于写操作，需使用 ActiveModel。
/// [DeriveEntityModel] 这个 derive 根据我们定义的[Model]，生成了包括 [ActiveModel]在内的多个数据结构
impl ActiveModelBehavior for ActiveModel {}