use std::sync::Arc;

use axum::extract::{Query, State};
use sea_orm::{ColumnTrait, Condition, EntityTrait, PaginatorTrait, QueryFilter, QueryOrder, QuerySelect};

use crate::entity::{article, category};
use crate::err::AppError;
use crate::handler::{get_conn, HtmlResponse, log_error, render};
use crate::param::ArticleParams;
use crate::Result;
use crate::state::AppState;
use crate::view;

pub async fn index(
    State(state): State<Arc<AppState>>,
    Query(params): Query<ArticleParams>,
) -> Result<HtmlResponse> {
    let handler_name = "article/index";
    let conn = get_conn(&state);

    // 构建条件: 所有未被删除的文章
    let condition = Condition::all().add(article::Column::IsDel.eq(false));
    let select = article::Entity::find().filter(condition);
    let record_total = select
        .clone()
        .count(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let page_size = 15u64;
    let page = 0u64;

    let page_total = f64::ceil(record_total as f64 / page_size as f64) as u64;
    let offset = page_size * page;

    let list = select.find_also_related(category::Entity)
        .order_by_desc(article::Column::Id)
        .limit(page_size)
        .offset(offset)
        .all(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let tpl = view::ArticlesTemplate {
        list,
        page_total,
        params,
    };
    render(tpl, handler_name)
}