use std::sync::Arc;

use axum::extract::{Path, Query, State};
use axum::Form;
use sea_orm::{ActiveModelTrait, ColumnTrait, Condition, EntityTrait, IntoActiveModel, ModelTrait, PaginatorTrait, QueryFilter, QueryOrder};
use sea_orm::ActiveValue::Set;

use crate::{form, Result, view};
use crate::entity::{article, category};
use crate::err::AppError;
use crate::handler::{get_conn, HtmlResponse, log_error, redirect, RedirectResponse, render};
use crate::param::{CategoryParams, DelParams};
use crate::state::AppState;

pub async fn index(
    State(state): State<Arc<AppState>>,
    Query(params): Query<CategoryParams>,
) -> Result<HtmlResponse> {
    let handler_name = "category/index";
    let conn = get_conn(&state);

    let page = params.page(); // 当前页码
    let page_size = params.page_size(); // 每页条数，默认15
    let mut select = category::Entity::find()
        .filter(
            Condition::all()
                .add_option(
                    params.keyword_opt().map(|n| category::Column::Name.contains(&n))
                )
                .add_option(params.is_del_opt().map(|n| category::Column::IsDel.eq(n)))
        );

    if let Some(ord) = params.order() {
        select = select.order_by(category::Column::Id, ord);
    }

    let paginator = select.paginate(conn, page_size);

    let page_total = paginator.num_pages().await.map_err(AppError::from)?;

    if page_total == 0 {
        let tpl = view::CategoryTemplate {
            categories: vec![],
            params,
            page_total,
        };
        return render(tpl, handler_name);
    }

    let categories = paginator
        .fetch_page(page)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let tpl = view::CategoryTemplate {
        categories,
        params,
        page_total,
    };
    render(tpl, handler_name)
}


pub async fn find(
    State(state): State<Arc<AppState>>,
    Path(id): Path<i32>,
) -> Result<String> {
    let id = u32::try_from(id).or(Err(AppError::notfound()))?;

    let handler_name = "category/find";
    let conn = get_conn(&state);

    let cate = category::Entity::find_by_id(id)
        .one(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    match cate {
        None => Err(AppError::notfound()),
        Some(cate) => Ok(format!("id: {}, 名称: {}", cate.id, cate.name)),
    }
}

pub async fn add_ui() -> Result<HtmlResponse> {
    let handler_name = "category/add_ui";
    let tpl = view::CategoryAddTemplate;
    render(tpl, handler_name)
}

pub async fn add(
    State(state): State<Arc<AppState>>,
    Form(frm): Form<form::CategoryForm>,
) -> Result<RedirectResponse> {
    let handler_name = "category/add";
    let conn = get_conn(&state);

    let am = category::ActiveModel {
        name: Set(frm.name),
        // all other attributes are `NotSet`
        ..Default::default()
    };

    let added_category = am.insert(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let url = format!("/category?msg=分类添加成功，ID是：{}", added_category.id);

    redirect(url.as_str())
}

pub async fn edit_ui(
    State(state): State<Arc<AppState>>,
    Path(id): Path<i32>,
) -> Result<HtmlResponse> {
    let id = u32::try_from(id).or(Err(AppError::notfound()))?;

    let handler_name = "category/edit_ui";
    let conn = get_conn(&state);

    let cate = category::Entity::find_by_id(id)
        .one(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?
        .ok_or(AppError::notfound())?;

    let tpl = view::CategoryEditTemplate { category: cate };
    render(tpl, handler_name)
}

pub async fn edit(
    State(state): State<Arc<AppState>>,
    Path(id): Path<u32>,
    Form(frm): Form<form::CategoryForm>,
) -> Result<RedirectResponse> {
    let id = u32::try_from(id).or(Err(AppError::notfound()))?;

    let handler_name = "category/edit";
    let conn = get_conn(&state);

    let cate = category::Entity::find_by_id(id)
        .one(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?
        .ok_or(AppError::notfound())?;

    let mut am = cate.into_active_model();
    am.name = Set(frm.name);

    am.update(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    redirect("/category?msg=分类修改成功")
}


pub async fn del(
    State(state): State<Arc<AppState>>,
    Path(params): Path<DelParams>,
) -> Result<RedirectResponse> {
    let handler_name = "category/del";
    let conn = get_conn(&state);
    let real = params.real.unwrap_or(false);

    let cate = category::Entity::find_by_id(params.id)
        .one(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?
        .ok_or(AppError::notfound())?;

    if real {
        cate.delete(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;
    } else {
        let mut am = cate.into_active_model();
        am.is_del = Set(true);
        am.save(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;
    }

    redirect("/category?msg=分类删除成功")
}

pub async fn articles(
    State(state): State<Arc<AppState>>,
    Path(id): Path<u32>,
    Query(params): Query<CategoryParams>,
) -> Result<HtmlResponse> {
    let handler_name = "category/articles";
    let conn = get_conn(&state);

    let cate = category::Entity::find_by_id(id)
        .one(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?
        .ok_or(AppError::notfound())
        .map_err(log_error(handler_name))?;

    let paginator = cate.find_related(article::Entity)
        .paginate(conn, params.page_size());

    let articles = paginator.fetch_page(params.page())
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let page_total = paginator
        .num_pages()
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let tpl = view::CategoryArticlesTemplate {
        params,
        page_total,
        category: cate,
        articles,
    };
    render(tpl, handler_name)
}